﻿using System;
using Neuron.Esb;
using System.Transactions;
using System.Xml.Serialization;

namespace Neuron.Esb.Sample
{
    [XmlRoot("TestRoot"), Serializable]
    public class TestSerialization
    {
        [XmlElement("Name")]
        public string Name { get; set; }
        [XmlElement("Age")]
        public int Age { get; set; }
        [XmlElement("Sex")]
        public string Sex { get; set; }
    }

    class Program
    {
        /// <summary>
        /// set the the name of the Subscriber
        /// </summary>
        const string SUBSCRIBER = "OrderClient";

        static void Main(string[] args)
        {

            // Create an instance of a subscriber.
            using (Subscriber subscriber = new Subscriber(SUBSCRIBER))
            {

                subscriber.OnOnline += new Channels.TopicEventHandler(subscriber_OnOnline);

                // Wire in the on receive event to receive messages in real time as they are published to the bus
                subscriber.OnReceive += OnReceive;

                // catch any exceptions that may occurr while connecting to each individual topic
                PartyConnectExceptions exceptions = subscriber.Connect();
                if (exceptions != null && exceptions.Count > 0)
                {
                    foreach (var e in exceptions.GetResults())
                        Console.WriteLine(string.Format("An error occurred connecting '{0}' to Topic, '{1}'. {2}", subscriber.Context.PartyId,e.Exception.Message, e.Topic));
                }

                
                Console.ReadLine();

            }
        }


        private static void subscriber_OnOnline(object o, TopicEventArgs e)
        {
            if (e != null) Console.WriteLine("  Online Event = " + e.Topic);
        }

        private static void OnReceive(object sender, MessageEventArgs e)
        {

            ESBMessage message = e.Message;

            // print out the message
            PrintOutMessageByType(message);

            /// <summary>
            /// This just shows that if you are using MSMQ as a transport, you can capture the ambiant transaction 
            /// and roll back the message to the underlying queue.  You can also just set e.Cancel = True
            /// This is currently just commented out
            /// </summary>
            //Transaction tx = System.Transactions.Transaction.Current;
            //if (tx == null) Console.WriteLine("    null transaction");
            //else Console.WriteLine("   transaction exists");

            //tx.Rollback(); 
           
          
            // send back the response if request
            if (message.Header.Semantic == Semantic.Request)
            {
                // create reply from original message so all the 
                // correlation properties and headers are set
                ESBMessage retVal = message.CreateReplyMessage();
                retVal.Body = "Response to intial request received: " + message.Text;

                // send using original subscriber object
                (sender as Subscriber).SendMessage(retVal);
            }

        }



        private static void PrintOutMessageByType(ESBMessage message)
        {
            // first print out the message
            if (message.Header.BodyType != null)
            {
                if (message.Header.BodyType.Equals("text/xml"))
                    Console.WriteLine(message.GetBody<string>());

                if (message.Header.BodyType.Equals("DateTime"))
                    Console.WriteLine(message.GetBody<DateTime>().ToString());

                if (message.Header.BodyType.Equals("TestSerialization"))
                    Console.WriteLine(message.GetBody<TestSerialization>().Name);
            }
            else
            {
                Console.WriteLine(message.Text);
            }
        }
    }
}
